/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.fsa.swing;

import java.awt.*;

import javax.swing.*;


/**
 * An extended JSeparator.
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.11 $
 */
public class JExtendedSeparator extends JSeparator
{
   /**
    * first component of the two components separated by the separator.
    */
   private Component first = null;


   /**
    * get the value of first.
    *
    * @return   the value of first
    */
   public Component getFirst()
   {
      return this.first;
   } // getFirst


   /**
    * set the value of first.
    *
    * @param value  The new first value
    */
   public void setFirst (Component value)
   {
      if (this.first != value)
      {
         this.first = value;
      } // if ()
   } // setFirst


   /**
    * the second component of the two components separated by the Separator.
    */
   private Component second = null;


   /**
    * get the value of second.
    *
    * @return   the value of second
    */
   public Component getSecond()
   {
      return this.second;
   } // getSecond


   /**
    * set the value of second.
    *
    * @param value  The new second value
    */
   public void setSecond (Component value)
   {
      if (this.second != value)
      {
         this.second = value;
      } // if ()
   } // setSecond


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private boolean showWithSecond;


   /**
    * Sets the showWithSecond attribute of the JExtendedSeparator object
    *
    * @param value  The new showWithSecond value
    */
   public void setShowWithSecond (boolean value)
   {
      if (this.showWithSecond != value)
      {
         this.showWithSecond = value;
      } // end of if ()
   }


   /**
    * Get the showWithSecond attribute of the JExtendedSeparator object
    *
    * @return   The showWithSecond value
    */
   public boolean isShowWithSecond()
   {
      return this.showWithSecond;
   }


   /**
    * Get the visible attribute of the JExtendedSeparator object
    *
    * @return   The visible value
    */
   public boolean isVisible()
   {
      if (!isShowWithSecond())
      {
         return super.isVisible();
      } // end of if ()
      else
      {
         return  (getSecond() != null &&
            getSecond().isVisible());
      } // end of else
   }

}

/*
 * $Log: JExtendedSeparator.java,v $
 * Revision 1.11  2004/11/03 10:17:57  lowende
 * Javadoc warnings removed.
 *
 */
