/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.layout.classdiag.internalmodel;

import java.awt.Dimension;


/**
 * Any layouter for any diagram type should implement this interface. The class
 * has been taken from Argo UML source code.
 *
 * @author    $Author: joerg $
 * @version   $Revision: 1.1 $ $Date: 2005/01/24 12:30:49 $
 */
public interface Layouter
{

   /**
    * Add another object to the diagram.
    *
    *
    * @param obj             represents the object to be part of the diagram.
    */
   void add (LayoutedObject obj);


   /**
    * Remove a object from the diagram.
    *
    *
    * @param obj             represents the object to be removed.
    */
   void remove (LayoutedObject obj);


   /**
    * Operation getObjects returns all the layouted objects from this diagram.
    *
    *
    * @return   An array with the layouted objects of this diagram.
    */
   LayoutedObject[] getObjects();


   /**
    * Operation getObject returns one object from the diagram.
    *
    *
    * @param index             represents the index of this object.
    * @return       the object
    */
   LayoutedObject getObject (int index);


   /**
    * This operation starts the actual layout process.
    */
   void layout();


   /**
    * Operation getMinimumDiagramSize returns the minimum diagram size after the
    * layout, so the diagram could be resized to this size.
    *
    *
    * @return   the size/dimension
    */
   Dimension getMinimumDiagramSize();
}

/*
 * $Log: Layouter.java,v $
 * Revision 1.1  2005/01/24 12:30:49  joerg
 * started new layout algoritm for class diagrams
 *
 */
